import re
import pickle
import json
import pandas as pd
from number_parser import parse
from price_parser import Price
from flashtext import KeywordProcessor


def word2number(series: pd.Series, lang='id') -> pd.Series:
    if lang == 'eng':
        return series.apply(parse)
    else:
        return series


def extract_hashtag(series: pd.Series) -> pd.DataFrame:
    list_text, list_hashtag = [], []
    for _, value in series.items():
        get_hashtag = list(
            set(part[1:] for part in value.split() if part.startswith('#')))
        if get_hashtag:
            list_text.append(value)
            list_hashtag.append(get_hashtag)

    return pd.DataFrame({'Text': list_text, 'Hashtag': list_hashtag})


def extract_price(series: pd.Series) -> pd.DataFrame:
    list_text, list_price = [], []
    for _, value in series.items():
        price = Price.fromstring(value)
        if price.currency in ['Rp', 'RP', '$']:
            list_text.append(value)
            list_price.append(price)

    return pd.DataFrame({'Text': list_text, 'Price': list_price})


def encode_email(series: pd.Series) -> pd.Series:
    series = series.replace(regex=r'[\w\.-]+@[\w\.-]+\.\w+', value='<EMAIL>')
    return series


def encode_date(series: pd.Series) -> pd.Series:
    ddmmyyyy = r'\b(3[01]|[12][0-9]|0[1-9])/(1[0-2]|0[1-9])/([0-9]{4})\b'
    yyyymmdd = r'\b([0-9]{4})/(1[0-2]|0[1-9])/(3[01]|[12][0-9]|0[1-9])\b'
    date = re.compile('|'.join([ddmmyyyy, yyyymmdd]))
    series = series.replace(regex=date, value='<DATE>')
    return series


def encode_phone_num(series: pd.Series) -> pd.Series:
    re_phone_num = r'(\+62\s?|0)(\d{3,4}-?){2}\d{3,4}'
    series = series.replace(regex=re_phone_num, value='<PHONE NUM>')
    return series


def read_json(json_path: str) -> dict:
    with open(json_path, 'r') as file:
        data_dict = json.load(file)
    return data_dict


def convert_slang_formal(series: pd.Series) -> pd.Series:
    dict_alay = read_json('maleo/preprocessing/slang_dict.json')

    keyword_proc = KeywordProcessor()
    for word in dict_alay.items():
        keyword_proc.add_keyword(word[0], word[1])

    series = pd.Series(series)
    result = series.apply(keyword_proc.replace_keywords)
    result = result.replace(r"\s{2,}", "")
    result = result.str.strip()
    return result


def convert_emojis_to_word(series: pd.Series) -> pd.Series:
    with open('maleo/preprocessing/Emoji_Dict.p', 'rb') as fp:
        emoji_dict = pickle.load(fp)
    emoji_dict = {v: k for k, v in emoji_dict.items()}

    for emot in emoji_dict:
        pattern = r'(' + emot + ')'
        val = "_".join(emoji_dict[emot].replace(
            ",", "").replace(":", "").split())
        series = series.replace(regex=pattern, value=val)
    return series



from maleo.stopword_remover.ArrayDictionary import ArrayDictionary
from maleo.stopword_remover.Remover import Remover


class RemoverFactory(object):
    """description of class."""

    def create_stop_word_remover(self):
        stopWords = self.get_stop_words()
        dictionary = ArrayDictionary(stopWords)
        stopWordRemover = Remover(dictionary)

        return stopWordRemover

    def get_stop_words(self):
        with open('maleo/stopword_remover/indo_stopwords.txt', 'r') as file:
            tmp = file.readlines()
            stopword = [word.rstrip('\n') for word in tmp]
        return stopword



import re
import pickle
import unicodedata
import pandas as pd

from bs4 import BeautifulSoup
from maleo.stopword_remover.RemoverFactory import RemoverFactory


def remove_multiple_space(series: pd.Series) -> pd.Series:
    series = series.replace(regex=r'\s\s+', value=' ')
    return series


def remove_link(series: pd.Series) -> pd.Series:
    series = series.replace(regex=r'http\S+', value='')
    series = series.replace(regex=r'pic.twitter.com\S+', value='')
    return series


def remove_punctuation(series: pd.Series) -> pd.Series:
    punctuation_pattern = r"[\s{}]".format(
        re.escape('!"#$%&\'()*+,-./:;=?@[\\]^_`{|}~'))
    series = series.replace(regex=punctuation_pattern, value=' ')
    return series


def remove_char(series: pd.Series) -> pd.Series:
    # remove single char
    series = series.replace(regex=r'\s[^uUgGbB0-9]\s', value=' ')
    # remove consecutive repeating char
    series = series.replace(regex=r'([^gG0-9])\1+', value=r'\1')
    return series


def remove_html(series: pd.Series) -> pd.Series:
    new_series = []
    for text in series:
        new_text = BeautifulSoup(text, "html.parser")
        new_series.append(new_text.get_text())
    return pd.Series(new_series)


def remove_non_ascii(series: pd.Series) -> pd.Series:
    new_series = []
    for text in series:
        new_text = unicodedata.normalize('NFKD', text).encode(
            'ascii', 'ignore').decode('utf-8', 'ignore')
        new_series.append(new_text)
    return pd.Series(new_series)


def remove_stopword(series: pd.Series) -> pd.Series:
    factory = RemoverFactory()
    stopword = factory.create_stop_word_remover()

    result = series.copy()
    for idx, row in series.items():
        result.iloc[idx, :] = stopword.remove(row)
    return result


def remove_emoticons(series: pd.Series) -> pd.Series:
    with open('maleo/cleansing/Emoticon_Dict.p', 'rb') as file:
        emoticon_dict = pickle.load(file)
    emoticon_pattern = re.compile(
        u'(' + u'|'.join(k for k in emoticon_dict) + u')')
    result = series.replace(regex=emoticon_pattern, value=r'')
    return result



from setuptools import setup, find_packages

with open('README.md', encoding='utf8') as f:
    long_description = f.read()

setup(
    name='maleo',
    packages=find_packages(),
    version='0.0.3',
    license='MIT',
    description='Wrapper library for data cleansing, preprocessing in text',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Ruben Stefanus',
    author_email='researchjair@gmail.com',
    url='https://github.com/jakartaresearch/maleo',
    download_url='https://github.com/jakartaresearch/maleo/archive/v0.0.3.tar.gz',
    keywords=['nlp', 'text-processing', 'machine-learning'],
    install_requires=[
        'pandas>=1.1.1',
        'emoji>=0.6.0',
        'nltk>=3.5',
        'number-parser>=0.2.1',
        'price-parser>=0.3.3',
        'flashtext>=2.7'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Text Processing',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



import pandas as pd

from maleo.scanner.scanner import scan
from maleo.cleansing.cleansing import remove_link, remove_punctuation, remove_char, remove_html
from maleo.cleansing.cleansing import remove_non_ascii, remove_stopword, remove_emoticons
from maleo.cleansing.cleansing import remove_multiple_space
from maleo.preprocessing.preprocessing import encode_email, encode_date, encode_phone_num
from maleo.preprocessing.preprocessing import convert_slang_formal, word2number, convert_emojis_to_word
from maleo.preprocessing.preprocessing import extract_price, extract_hashtag


class Wizard:
    def __init__(self):
        self.scanner = scan
        self.rm_multiple_space = remove_multiple_space
        self.rm_link = remove_link
        self.rm_punc = remove_punctuation
        self.rm_char = remove_char
        self.rm_html = remove_html
        self.rm_non_ascii = remove_non_ascii
        self.rm_stopword = remove_stopword
        self.rm_emoticon = remove_emoticons
        self.word_to_number = word2number
        self.get_hashtag = extract_hashtag
        self.get_price = extract_price
        self.email_to_tag = encode_email
        self.date_to_tag = encode_date
        self.phone_num_to_tag = encode_phone_num
        self.slang_to_formal = convert_slang_formal
        self.emoji_to_word = convert_emojis_to_word

    def convert_dtype(self, text) -> pd.Series:
        if isinstance(text, list) or isinstance(text, str):
            text = pd.Series(text)
        elif not isinstance(text, pd.core.series.Series):
            print('Data Type not supported')
        return text
